# Support stuff for Greasemonkey bundle commands for TextMate
# By Henrik Nyh <http://henrik.nyh.se> 2007-03-02.
# Free to modify and redistribute non-commercially with due credit.


class NotInstalledScriptException < RuntimeError; end

InstalledScript = Struct.new(:name, :filename)


class AppleScript
	def self.ensure_gui_scripting(mess)
		%{
			tell application "System Events"
				if not UI elements enabled then
					do shell script "osascript \\"#{ENV['TM_BUNDLE_SUPPORT']}/bin/enable_assistive_devices.app\\" \\"#{mess}\\""
					return
				end if
			end tell
		}
	end
end


class MyTextMate
	def self.open(file, action=:run)
		command = "open -a TextMate #{e_sh file}"
		if (action == :output_command) then command else `#{command}` end
	end
end


class Greasemonkey

	class Script
		attr_reader :file_path, :file_name, :file_lines, :file_data, :name, :namespace, :description, :includes, :excludes
		def initialize(config)
			@config = config
			@file_path = ENV['TM_FILEPATH']
			@file_name = File.basename(@file_path) if @file_path
			
			# These values are only present if the command input is "Entire document"!
			@file_lines = STDIN.readlines
			@file_data = @file_lines.join
			
			declaration_value = /[ \t]+(\S.*?)\s*[\r\n]/
			@name = ($1 if @file_data =~ /@name#{declaration_value}/)
			@namespace = ($1 if @file_data =~ /@namespace#{declaration_value}/)
			@description = ($1 if @file_data =~ /@description#{declaration_value}/) || ""
			@includes = @file_data.scan(/@include#{declaration_value}/).flatten
			@excludes = @file_data.scan(/@exclude#{declaration_value}/).flatten
			@includes = ["*"] if @includes.empty?
		end
		def ensure_valid
			abort "This script does not have a @name and a @namespace!" unless (@name and @namespace)
		end
		def ensure_saved
			abort "This script is not saved!" if (@file_path.nil? or @file_path.empty?)
		end
		def ensure_installed
			@config.ensure_existence
			abort "This file is not an installed script!" unless File.dirname(@file_path.to_s) == @config.directory
		end
		def avoid_installed
			@config.ensure_existence
			abort "This script is already installed!" if File.dirname(@file_path.to_s) == @config.directory
		end
	end
	
	
	class Config
		attr_reader :directory, :file_name
		def initialize
			firefox_support = "#{ENV['HOME']}/Library/Application Support/Firefox"
			@directory = "#{firefox_support}/#{$1}/gm_scripts" if File.open("#{firefox_support}/profiles.ini").read =~ /Path=(\S+)\s*(?:Default=1|\Z)/
			@file_name = "#{@directory}/config.xml" if @directory
		end
		def file_lines
			File.open(@file_name).to_a
		end
		def ensure_existence
			abort "Could not locate Firefox directory!" unless File.exists?(@file_name.to_s)
		end
		def xml
			self.ensure_existence
			
			require "rexml/document"
			xml_data = REXML::Document.new File.new(@file_name)
			
			if block_given?
				yield(xml_data)
				self.xml = xml_data
			else
				xml_data
			end
		end
		def xml=(data)
			data.write File.open(@file_name, 'w')
		end
	end


	def initialize
		@config = Greasemonkey::Config.new
		@script = Greasemonkey::Script.new(@config)
	end

end